@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 """
 contants for Apoo
 
@@ -103,14 +103,20 @@ class FileError(vpuError):
     def __init__(self,line=0):
         self.message = 'File Error'
         
+class MemoryUnderflow(vpuError):
+    def __init__(self,add):
+        self.message = 'Memory Underflow'
+        self.add = add
+        self.colour = 'red'
+
 # zero arg, nonreg, reg, reg reg, nonreg reg, reg nonreg, specials
 inst = (['rtn','halt','nop'], # zero arg
         ['jsr','jump'], # nonreg
         ['inc','dec','zero','not','jumpi','push','pop'], #reg
         ['storei','loadi','storer','add','sub','mul','div','mod',
          'and','or','xor'], # reg reg
-        ['load','loadn'], # nonreg reg
-        ['store','jzero','jnzero','jpos','jneg'], # reg nonreg
+        ['load','loadn','loado'], # nonreg reg
+        ['store','jzero','jnzero','jpos','jneg','storeo'], # reg nonreg
         ['mem','const','string','equ']) #specials
 
 
