@@ -1,8 +1,9 @@
+#!/usr/bin/python
 # -*- coding: utf-8 -*-
 """
 Apoo Virtual Processor
 
-Copyright (C) 1998-2003 Rogério Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
+Copyright (C) 1998-2006 Rogério Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,12 +25,12 @@ from constants import *
 import string, sys, time, copy
 
 
-__version = "$Id: vpu.py,v 1.49 2006-04-14 21:21:00 rvr Exp $"
-
+__version = "3.0p2"
+Changed = (None,None)
 
 
 class Vpu:
-    def __init__(self, n=8, MpMem = {},Inter=None):
+    def __init__(self, n=8, MpMem={}, Inter=None, RAMSize=1000):
         """This is where all the pseudo-code is specified. Any new
         intruction added shoud be added to the constants.py module
         too, where the lexical information resides.  @arg n: is the
@@ -44,11 +45,14 @@ class Vpu:
         self.labelms = {}
         self.constants = {}
         self.PC = 0
+        self.SPn = n-1
+        self.FPn = n-2
         self.BreakP = []
-        self.stack = Stack()
+        self.StaticMem = 0
         self.lines = []
         self.time0, self.time = 0,0
         self.Inter=Inter
+        self.RAMSize = RAMSize
         self.MpMem = copy.deepcopy(MpMem)
         self.code = {'add':("Reg[A2] = Reg[A2] + Reg[A1]",
                             "incPC()"),
@@ -76,7 +80,7 @@ class Vpu:
                      'jsr':("if type(A1) == type(''):",
                             " add = self.ParseLabelP(A1)",
                             "else: add = A1",
-                            "self.stack.push(self.PC + 1)",
+                            "self.push(self.PC + 1)",
                             "self.PC = add"),
                      'jump':("if type(A1) == type(''):",
                              " try: add = self.labelp[A1]",
@@ -114,17 +118,22 @@ class Vpu:
                               "else: v = A1",
                               "Reg[A2] = v",
                               "incPC()"),
+                     'loado':("add = Reg[self.FPn]+A1",
+                              "try: foo = self.MLoad(add)",
+                              "except IndexError: raise OutOfMemory(add)",
+                              "Reg[A2] = foo",
+                              "incPC()"),
                      'mod':("Reg[A2] = Reg[A1] % Reg[A2]",
                             "incPC()"),
                      'mul':("Reg[A2] = Reg[A1] * Reg[A2]",
                             "incPC()"),
                      'or':("Reg[A2] = Reg[A1] | Reg[A2]",
                            "incPC)"),
-                     'pop':("Reg[A1] = self.stack.pop()",
+                     'pop':("Reg[A1] = self.pop()",
                             "incPC()"),
-                     'push':("self.stack.push(Reg[A1])",
+                     'push':("self.push(Reg[A1])",
                              "incPC()"),
-                     'rtn':("self.PC = self.stack.pop()"),
+                     'rtn':("self.PC = self.pop()"),
                      'store':("if type(A2) == type (''):",
                               " try: add = self.labelm[A2]",
                               " except KeyError: raise LabelError",
@@ -142,6 +151,11 @@ class Vpu:
                                "else: r = A1",
                                "Reg[A2]=r",
                                "incPC()"),
+                     'storeo':("add = Reg[self.FPn]+A2",
+                               "try:self.MStore(add,Reg[A1])",
+                               "except IndexError:",
+                               " raise OutOfMemory(add)",
+                               "incPC()"),
                      'storer':("Reg[A2] = Reg[A1]",
                                "incPC()"),
                      'sub':("Reg[A2] = Reg[A1] - Reg[A2]",
@@ -164,13 +178,12 @@ class Vpu:
         self.labelms = {}
         self.PC = 0
         self.BreakP = []
-        self.stack = Stack()
         self.lines = []
         self.time0, self.time = 0,0
 
     def __str__(self):
         """Only used for debugging purposes."""
-        return str((self.PC,self.reg,self.stack))
+        return str((self.PC,self.reg,self.reg[self.SPn]))
 
     def __repr__(self):
         """Only used for debugging purposes."""
@@ -178,19 +191,26 @@ class Vpu:
 
     def MStore(self,add,val):
         """Store in memory shell that deals with mapped memory """
-        if add not in self.MpMem.keys():
-            self.RAM[add] = val
-        else:
+        global Changed
+        if add in self.MpMem.keys():
             exec self.MpMem[add][1]
+        elif self.RAMSize <= add or add < 0:
+            raise OutOfMemory(add)
+        else:
+            self.RAM[add] = val
+            Changed = (add,val)
+
         
     def MLoad(self,add):
         """Load from RAM shell that deals with mapped memory """
-        if add not in self.MpMem.keys():
-            return self.RAM[add]
-        else:
+        if add in self.MpMem.keys():
             val = 0
             exec self.MpMem[add][0]
             return val
+        elif self.RAMSize <= add or add < 0:
+            raise OutOfMemory(add)
+        else:
+            return self.RAM[add]
         
     def run(self, MaxSteps=1000):
         """Starts the execution of the current program.
@@ -198,8 +218,8 @@ class Vpu:
         @arg MaxSteps: the maximum allowed number of steps to execute
         for infinite loop detection."""
         i = 0
-#        self.PC = 0
-        while self.step() == 1:
+        while True:
+            self.step()
             if i > MaxSteps: raise TooManySteps(i)
             else: i = i + 1
 
@@ -215,7 +235,8 @@ class Vpu:
 
     def cont(self,num):
         i = 0
-        while self.step() == 1:
+        while True:
+            self.step()
             if i > num: raise TooManySteps(num)
             i = i + 1
             if self.PC in self.BreakP:
@@ -223,6 +244,8 @@ class Vpu:
 
     def step(self):
         """basic execution of a step of the program"""
+        global Changed
+        Changed=(None,None)
         self.TimerOn()
         try:
             i = self.Prog[self.PC]
@@ -234,7 +257,7 @@ class Vpu:
         else:
             exec self.code[i[0]]
         self.TimerOff()
-        return 1
+        return Changed
     
     def incPC(self):
         self.PC = self.PC + 1
@@ -324,8 +347,22 @@ class Vpu:
                 self.Prog.append((i[1],r1,self.ParseNum(i[3])))
             else:
                 raise IllInst(n)
-            
-
+        self.StaticMem = len(self.RAM)-1
+        self.RAM += [ 0 for i in xrange(self.RAMSize)]
+        self.reg[self.SPn] = self.StaticMem
+        self.reg[self.FPn] = self.reg[self.SPn] + 1
+        
+    def push(self,val):
+        self.reg[self.SPn] += 1
+        self.MStore(self.reg[self.SPn],val)
+        
+    def pop(self):
+        if self.reg[self.SPn] <= self.StaticMem:
+            raise MemoryUnderflow(self.reg[self.SPn])
+        foo = self.MLoad(self.reg[self.SPn])
+        self.reg[self.SPn] -= 1
+        return foo
+        
     def ParseNum(self,st):
         if st in self.constants.keys():
             return self.constants[st]
@@ -392,62 +429,6 @@ class Vpu:
     def TimerOff(self):
         self.time = self.time + (time.clock() - self.time0)
 
-
-error = 'stack.error'
-
-class Stack:
-    """ A class to implement stacks """
-    def __init__(self, start=[]):
-        self.stack = []
-        for x in start: self.push(x)
-        self.stack.reverse()                   # reverse initial stack objects
-
-    def push(self, object):
-        self.stack = [object] + self.stack
-
-    def pop(self):
-        if not self.stack:                   # stack empty!
-            raise 'stack underflow'
-        top, self.stack = self.stack[0], self.stack[1:]
-        return top
-
-    def top(self):
-        if not self.stack:                   # stack empty!
-            raise 'stack underflow'
-        return self.stack[0]
-
-    def empty(self):
-        return not self.stack
-
-    def popall(self):
-        """empties stack"""
-        self.stack=[]
-        
-    def __repr__(self):
-        return '[Stack:%s]' % self.stack
-
-    def __cmp__(self, other):
-        return cmp(self.stack, other.stack)  # what the heck we are just lists
-
-    def __len__(self):
-        return len(self.stack)
-
-    def __add__(self, other):                # inst1 + inst2
-        return Stack(self.stack + other.stack)
-
-    def __mul__(self, reps):                 # inst * reps
-        return Stack(self.stack * reps)
-
-    def __getitem__(self, offset):
-        return self.stack[offset]
-
-    def __getslice__(self, low, high):
-        return Stack(self.stack[low : high])
-
-    def __getattr__(self, name):             # inst.sort(), inst.reverse()
-        return getattr(self.stack, name)
-
-    
 def ReadProgram(filename):
     try:
         input = open(filename, 'r')
@@ -483,13 +464,16 @@ def CantRead(file):
 def ParseReg(st,nreg,line):
     if st[0] != 'r' and st[0] != 'R':
         raise IllOperand(line)
-    else:
-        try: i = int(st[1:len(st)])
-        except ValueError:
-            raise IllOperand(line)
-        if not i in range(nreg):
-            raise IllReg(line)
-        return i
+    elif st == "RS" or st == "rs":
+        st = "R%d"%(nreg-1)
+    elif st == "RF" or st == "rf":
+        st = "R%d"%(nreg-2)
+    try: i = int(st[1:len(st)])
+    except ValueError:
+        raise IllOperand(line)
+    if not i in range(nreg):
+        raise IllReg(line)
+    return i
 
 def expandCode(list):
     if type(list) == type(''):
